#
# Simple Python library for Deltronics Control and Datacapture Interface
# and Control It Extra
#
# USB communication via PyUSB library
#

import sys, usb.core, usb.util

device = None
outputs  = 0
motors = 0
motorspeeds = [0,0,0,0]
devicetype = 0

def initialize():
	# find usb device
	global device, devicetype

	# look for control and capture interface
	device = usb.core.find(idVendor=0x139C, idProduct=120)

	# was it found ?
	if device is None:
		# look for control it extra
		device = usb.core.find(idVendor=0x139C, idProduct=140)
		devicetype = 1
		if device is None:
			raise ValueError('Device not found')

	try:
		if device.is_kernel_driver_active(0):
			device.detach_kernel_driver(0)
	except usb.core.USBError as e:
		sys.exit("Kernel driver won't give up control over device: %s" % str(e))


	try:
		device.set_configuration()
		device.reset()
	except usb.core.USBError as e:
		sys.exit("Cannot set configuration the device: %s" % str(e))

def read_raw_data():
	global device
	endpoint = device[0][(0,0)][0]
	data = device.read(endpoint.bEndpointAddress,endpoint.wMaxPacketSize)
	return data

def write_data():
	global outputs, motors, motorspeeds, device
	if devicetype == 1:
		outputlow = outputs&0xFF
		outputhigh = (outputs&0xFF00)>>8
		data = [outputlow, motors, motorspeeds[0], motorspeeds[1], motorspeeds[2], motorspeeds[3], outputhigh, 0]
	else:
		data = [outputs, motors, motorspeeds[0], motorspeeds[1], motorspeeds[2], motorspeeds[3], 0, 0]
	device.ctrl_transfer(0x21, 0x09, 0x0200, 0, data, 100)
 

def setoutput( ioutput, value ):
# helper function for the set_output and set_outputs
	global outputs
	if ( devicetype == 1 ):
		numOutputs = 16
	else:
		numOutputs = 6

	if ( ioutput >=0 and ioutput < numOutputs ):
		outputs = outputs & ~(1<<ioutput)
		if value:
			outputs = outputs | (1<<ioutput)

def set_output( ioutput, value ):
	setoutput( ioutput, value )
	write_data()

def set_outputs( vallist ):
	i = 0
	for val in vallist:
		setoutput( i, val )
		i += 1
	write_data()

def clear_outputs():
	global outputs
	outputs = 0
	write_data()

def set_motor_speed( imotor, speed ):
	global motorspeeds
	if ( imotor >= 0 and imotor < 4 ):
		motorspeeds[imotor] = speed
		write_data()

def set_motor( imotor, onoff ):
	global motors
	if ( imotor >=0  and imotor < 4 ):
		motors = motors & ~(1<<(2*imotor))
		if ( onoff ):
			motors = motors | (1<<(2*imotor))
		write_data()

def set_motor_direction( imotor, direction ):
	global motors
	if ( imotor >=0  and imotor < 4 and (direction == 0 or direction == 1 ) ):
		motors = motors & ~(1<<(1+2*imotor))
		motors = motors | (direction<<(1+2*imotor))
		write_data()

def get_inputs():
	rawdata = read_raw_data()
	dataList = []
	for i in range( 0, 6 ):
		if (rawdata[0]&(1<<i) == 0):
			dataList.append( False )
		else:
			dataList.append( True )
	if ( devicetype == 1 ):
		# control it extra - add the last two bits of the low byte and the high byte inputs
		for i in range( 6, 8 ):
			if (rawdata[0]&(1<<i) == 0):
				dataList.append( False )
			else:
				dataList.append( True )

		for i in range( 0, 8 ):
			if (rawdata[16]&(1<<i) == 0):
				dataList.append( False )
			else:
				dataList.append( True )
		
	return dataList

def get_input( iinput ):
	allInputs = get_inputs()
	return allInputs[iinput]

#def get_analogue_data( port ):
#	rawdata = read_raw_data()
#
#	value = (rawdata[2+port*2]<<8) | rawdata[2*port*2+1]
#	return value

def get_analogue_sensor_id( port ):
	rawdata = read_raw_data()
	return rawdata[10+port]

def get_analogue_data( port ):
        rawdata = read_raw_data()
        # values are between 0 and 255 it seems
        #value = (rawdata[2+port*2]<<8) | rawdata[2*port*2+1]
        value = (rawdata[2+(port*2)]<<8) | rawdata[(2+(port*2)+1)]
        return value

